import abc
from ec2init.execution.procedure import Procedure
from typing import List, Any

class AsyncProcedure(Procedure):
    
    def is_async(self) -> bool:
        return True

    @property
    def max_retries(self) -> int:
        if hasattr(self, "_max_retries"):
            return self._max_retries
        return 5
    
    @max_retries.setter
    def max_retries(self, max_retries: int) -> None:
        self._max_retries = max_retries

    @property
    def max_workers(self) -> int:
        if hasattr(self, "_max_workers"):
            return self._max_workers
        return 5

    @max_workers.setter
    def max_workers(self, max_workers: int) -> None:
        self._max_workers = max_workers

    @property
    def tasks(self) -> List[Any]:
        if hasattr(self, "_tasks"):
            return self._tasks
        return []

    @tasks.setter
    def tasks(self, tasks: List[Any]) -> None:
        self._tasks = tasks

    @abc.abstractmethod
    def wait_to_finish(self) -> None:
        pass

    @abc.abstractmethod
    def cancel(self) -> None:
        pass
    
    @abc.abstractmethod
    def check_unhandled_exception(self) -> None:
        pass

    @abc.abstractmethod
    def is_running(self) -> bool:
        pass
    
    @abc.abstractmethod
    def completed_callback(self) -> bool:
        pass
