import abc
from ec2init.execution.base_async_procedure import BaseAsyncProcedure
from ec2init.execution.procedure import Procedure

from concurrent.futures import ProcessPoolExecutor, Executor
from typing import List, Callable, Optional, Any
import logging
import os
import pathlib
import time

logger = logging.getLogger(__name__)

class MultiProcessProcedure(BaseAsyncProcedure):

    def create_pool(self) -> ProcessPoolExecutor:
        pool = ProcessPoolExecutor(max_workers=self.max_workers)
        _check_file = MultiProcessProcedure._get_check_file_running(self.name)
        if not _check_file.is_file():
            _check_file.touch()
        return pool
    
    def submit_task(self, pool :Executor, task :Any):
        if self.task_function is not None:
            return pool.submit(self.task_function, task)
        raise NotImplementedError("task_function is not found")

    @property
    def task_function(self) -> Optional[Callable]:
        if hasattr(self, "_task_function"):
            return self._task_function
        return None 
    
    @task_function.setter
    def task_function(self, task_function:  Callable) -> None:
        self._task_function = task_function

    
    def post_pool_shutdown(self) -> None:
        _check_file = MultiProcessProcedure._get_check_file_running(self.name)
        if _check_file.is_file():
            os.remove(_check_file)

    def is_running(self) -> bool:
        return self.is_already_running(self.name)
    
    @staticmethod
    def is_already_running(name :str) -> bool:
        return MultiProcessProcedure._get_check_file_running(name).is_file()
    
    @staticmethod
    def _get_check_file_running(name: str) -> pathlib.Path:
        return pathlib.Path("/", "tmp", name + ".running")