from ec2init.utils.hopsfs import HopsFSUtils
from pathlib import Path
from ec2init.utils.bucket_configuration import BucketConfiguration
from ec2init import Procedure
import logging
from typing import List

logger = logging.getLogger(__name__)

class FixHhdfSiteForGCP(Procedure):
    def __init__(self, name: str, check_file: Path, bucket_configuration: BucketConfiguration, zone: str, deleted_blocks_retention_days:int) -> None:
        self.name = name
        self.check_file = check_file
        self.bucket_configuration = bucket_configuration
        self.zone = zone
        self.deleted_blocks_retention_days = deleted_blocks_retention_days

    def pre(self) -> None:
        pass

    def fix_hdfs_site_for_gcp(self, bucket: str, zone: str, backup_enabled: bool) -> None:
        HopsFSUtils.fix_hdfs_site("dfs.replication", "1")
        HopsFSUtils.fix_hdfs_site("dfs.enable.cloud.persistence", "true")
        HopsFSUtils.fix_hdfs_site("dfs.cloud.provider", "GCS")
        HopsFSUtils.fix_hdfs_site("dfs.cloud.gcs.bucket", bucket)
        HopsFSUtils.fix_hdfs_site("dfs.cloud.gcs.bucket.location", zone)
        HopsFSUtils.fix_hdfs_site("gcs.bucket.enable.versioning", str(backup_enabled)); 

    def execute(self) -> List[Procedure]:
        try :
            #add gcp related properties to hdfs-site.xml
            self.friendly_name = "Unable to set GCP related properties in hdfs-site.xml"
            self.fix_hdfs_site_for_gcp(self.bucket_configuration.name, self.zone, self.deleted_blocks_retention_days > 0)

            #Every thing is OK. 
            if self.check_file is not None:
                self.check_file.touch()

        except Exception as ex:
            logger.warn("%s - Could not update hdfs-site params for GCP. Container: %s %s", self.name, self.bucket_configuration.name, ex)
            raise Exception(self.friendly_name) 

        return []

    def rollback(self) -> None:
        pass