from ec2init.plans.plan_utils import PlanUtils
import logging
import configparser

from pathlib import Path
from typing import List

from ec2init import Procedure
import time 

logger = logging.getLogger(__name__)

class WaitForPrimaryHeadToStart(Procedure):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, gateway :str, user_id :str, instance_id :str, api_key :str) -> None:
        self.name = name 
        self.config = config
        self.check_file = check_file
        self.gateway = gateway
        self.user_id = user_id
        self.instance_id = instance_id
        self.api_key = api_key

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)

        master_is_running = False 
        while not master_is_running:
            cluster_status = PlanUtils.get_cluster_status(self.name, self.config, self.gateway, self.user_id, self.instance_id, self.api_key)
            logger.info("%s - got cluster status %s", self.name, str(cluster_status))
            master_is_running = len([node for node in cluster_status["payload"] if node["type"] == "master" and node["state"] == "running"]) == 1
            logger.info("%s - master_is_running %s", self.name, str(master_is_running))
            if master_is_running:
                break
            time.sleep(20)

        if self.check_file:
            self.check_file.touch()

        return []

    def rollback(self) -> None:
        pass
