import logging
import pathlib

from typing import List

from ec2init import Procedure
from ec2init.utils.system import SystemUtils
from pathlib import Path

logger = logging.getLogger(__name__)

class SetupCloudWatchAgent(Procedure):
    CONFIG_FILE = "/opt/aws/amazon-cloudwatch-agent/bin/aws_agent_config.json"

    @staticmethod
    def is_configured() -> bool:
        return pathlib.Path(SetupCloudWatchAgent.CONFIG_FILE).exists()

    def __init__(self, name: str, check_file: Path) -> None:
        self.name = name
        self.check_file = check_file

    def execute(self) -> List[Procedure]:
        logger.info(f"{self.name} - Starting execution")
        cmd = f"/opt/aws/amazon-cloudwatch-agent/bin/amazon-cloudwatch-agent-ctl -a fetch-config -m ec2 -s -c file:{SetupCloudWatchAgent.CONFIG_FILE}"
        SystemUtils.execute_command(cmd.split(" "))
        
        if self.check_file is not None:
            self.check_file.touch()
        return []

    def rollback(self):
        pass
