import logging
import importlib
from configparser import ConfigParser
from pathlib import Path

from typing import Any, Tuple

logger = logging.getLogger(__name__)

class CommonUtils:

    @staticmethod
    def load_class(canonical_class_name: str) -> Any:
        """
        Dynamically load a class given its canonical name
        """
        tokenized = canonical_class_name.split(".")
        module_name = ".".join(tokenized[:len(tokenized) - 1])
        class_name = tokenized[len(tokenized) - 1]
        logger.debug("Loading class: %s", canonical_class_name)

        module = importlib.import_module(module_name)
        return getattr(module, class_name)

    @staticmethod
    def _get_agent_credentials(config: ConfigParser) -> Tuple[str, str]:
        kagent_conf_dir = config.get('general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        kagent_conf_file = Path(kagent_conf_dir, 'config.ini')
        kagent_conf = ConfigParser()
        kagent_conf.read(str(kagent_conf_file))
        username = kagent_conf.get('server', 'username', fallback='agent@hops.io')
        password = kagent_conf.get('server', 'password', fallback='')
        return username, password