import abc

from typing import Dict, List, Optional, cast
from ec2init.utils.http import HttpUtils
import subprocess
from ec2init.utils.system import SystemUtils
from configparser import ConfigParser
from ec2init.utils.constants import Scripts

LOGIN_URI = (
    "POST", "https://hopsworks.glassfish.service.consul:8182/hopsworks-api/api/auth/login")
REFRESH_VARIABLE_URI = (
    "POST", "https://hopsworks.glassfish.service.consul:8182/hopsworks-api/api/admin/variables/refresh")

class VariablesStatement(abc.ABC):

    def __init__(self, id: str, value: str, visibility: int = 0, hide: bool = True) -> None:
        self.id = id
        self.value = value
        self.visibility = visibility
        self.hide = hide

    @abc.abstractmethod
    def statement(self) -> str:
        pass

class databaseUtils:

    ADMIN_EMAIL = "admin@hopsworks.ai"

    @staticmethod
    def _login(user_email: str, user_password: Optional[str], config: ConfigParser):
        if not user_email or not user_password:
            raise Exception("Admin username or password is not set")

        method, uri = LOGIN_URI
        data = {
            'email': user_email,
            'password': user_password,
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }

        response: Dict[str, str] = HttpUtils.do_plain_call(
            config, uri, method, data=data, headers=headers, verify=False)
        response_headers: Dict[str, str] = cast(
            Dict[str, str], response['headers'])

        jwt = response_headers['Authorization']
        return jwt

    @staticmethod
    def refresh_variables(config: ConfigParser, jwt: str):

        method, uri = REFRESH_VARIABLE_URI

        headers = {
            'Content-Type': 'application/json',
            'Authorization': jwt
        }

        HttpUtils.do_api_call(config, uri, method, headers=headers, verify=False)
        
    @staticmethod
    def sysexec(cmd: List[str], timeout: int = 120) -> subprocess.CompletedProcess:
        return SystemUtils.execute_command(cmd, timeout=timeout)

    @staticmethod
    def update_variable(id: str, value: str):
        databaseUtils.execute_variables_query(UpdateVariables(id, value))

    @staticmethod
    def run_command(cmd: List[str]) -> subprocess.CompletedProcess:
        cmd.insert(0, Scripts.MYSQL_CLIENT)
        return databaseUtils.sysexec(cmd)
    
    @staticmethod
    def execute_variables_query(statement: VariablesStatement) -> subprocess.CompletedProcess:
        cmd = [Scripts.MYSQL_CLIENT, "hopsworks", "-e", statement.statement()]
        return databaseUtils.sysexec(cmd)

    @staticmethod
    def insert_variables_db_cmd(id: str, value: str, visibility: int = 0) -> List[str]:
        return [Scripts.MYSQL_CLIENT, "hopsworks", "-e", databaseUtils.insert_variables_stmt(id, value, visibility)]
    
    @staticmethod
    def insert_variables_stmt(id: str, value: str, visibility: int = 0, hide: bool = False) -> str:
        return f"INSERT INTO variables (id, value, visibility, hide) VALUES ('{id}', '{value}', {visibility}, {hide})"
    

    @staticmethod
    def run_delete_statement(table: str, condition: str) -> None:
        databaseUtils.sysexec([Scripts.MYSQL_CLIENT, "hopsworks", "-e", f"DELETE from {table} where {condition}"])


class UpdateVariables(VariablesStatement):
    def statement(self) -> str:
        return f"UPDATE variables SET value='{self.value}' WHERE id='{self.id}'"

class ReplaceVariables(VariablesStatement):
    def statement(self) -> str:
        return f"REPLACE INTO variables (id, value, visibility, hide) VALUES ('{self.id}', '{self.value}', {self.visibility}, {self.hide})"
    
class InsertVariables(VariablesStatement):
    def statement(self) -> str:
        return f"INSERT INTO variables (id, value, visibility, hide) VALUES ('{self.id}', '{self.value}', {self.visibility}, {self.hide})"
    
class DeleteVariables(VariablesStatement):
    def __init__(self, id: str) -> None:
        super().__init__(id, '')

    def statement(self) -> str:
        return f"DELETE FROM variables WHERE id = '{self.id}'"