import logging
import xml.etree.ElementTree as ET
from xml.dom import minidom
from ec2init.utils.shell import ShellUtils
from os import path

logger = logging.getLogger(__name__)


class HopsFSUtils:
    @staticmethod
    def fix_hdfs_site(prop_name:str , prop_value:str ) -> None:
        HopsFSUtils.update_site_xml(prop_name, prop_value, "/srv/hops/hadoop/etc/hadoop/hdfs-site.xml")
    
    @staticmethod
    def update_site_xml(prop_name :str , prop_value :str, site_xml :str) -> None:
        found = False
    
        tree = ET.parse(site_xml)
        root = tree.getroot()
    
        for p in root.iter('property'):
            p_name = p.find("name")
            if p_name is not None and p_name.text == prop_name:
                logger.warn("%s is already defined in the %s with value: %s. New value: %s ", prop_name, site_xml, p[1].text, prop_value)
                p_value = p.find("value")
                if p_value is not None:
                    p_value.text = prop_value #overwrite the value
                found = True
                tree.write(open(site_xml, 'w'), encoding='unicode')
                break
        
        if not found:
            new_child = ET.Element("property")
            new_child.append(ET.Element("name"))
            new_child.append(ET.Element("value"))
            new_child[0].text = prop_name 
            new_child[1].text = prop_value 
            root.append(new_child)
            xmlstr = HopsFSUtils.prettry_print_xml(ET.tostring(root))
            with open(site_xml, "w") as f:
                f.write(xmlstr)
            logger.info("Added %s property to %s. Value: %s", prop_name, site_xml, prop_value)

    @staticmethod
    def prettry_print_xml(rawdata :bytes) -> str:
        return '\n'.join([line for line in minidom.parseString(rawdata).toprettyxml(indent=' '*2).split('\n') if line.strip()])

    @staticmethod
    def empty_datanode() -> None:
        dn_disk = "/hopsworks_data/hops/hopsdata/hdfs/dn/disk"
        dn_cloud = "/hopsworks_data/hops/hopsdata/hdfs/dn/cloud"
        if path.exists(dn_disk):
            ShellUtils.rmtree(dn_disk, True)
        if path.exists(dn_cloud):
            ShellUtils.rmtree(dn_cloud, True)